"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSourceFiles = getSourceFiles;
exports.addStoryGroupParam = addStoryGroupParam;
exports["default"] = _default;

var _parser = require("@babel/parser");

var _generator = _interopRequireDefault(require("@babel/generator"));

var _estraverse = _interopRequireDefault(require("estraverse"));

var _shared = require("./shared");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function getSourceFiles(ast) {
  var sourceFiles = [];
  var index = 0;

  _estraverse["default"].traverse(ast, {
    fallback: 'iteration',
    enter: function enter(node) {
      if (node.type === 'ImportDeclaration') {
        sourceFiles.push({
          path: node.source.value.replace(/^.*!/, ''),
          id: _shared.SOURCE_FILE_IMPORT_IDENTIFIER_PREFIX + index
        });
        index++;
      }
    }
  });

  return sourceFiles;
}

function addStoryGroupParam(ast) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$sourceFiles = _ref.sourceFiles,
      sourceFiles = _ref$sourceFiles === void 0 ? [] : _ref$sourceFiles;

  // Moves the default export declaration to a variable
  _estraverse["default"].replace(ast, {
    fallback: 'iteration',
    enter: function enter(node) {
      if (node.type === 'Program' && node.body) {
        // Finds the default export statement
        var exportDefaultDeclarationIndex = node.body.findIndex(function (childNode) {
          return childNode.type === 'ExportDefaultDeclaration';
        });

        if (~exportDefaultDeclarationIndex) {
          var _node$body;

          var exportDefaultDeclarationNode = node.body[exportDefaultDeclarationIndex]; // Copies the default export declaration to a variable

          node.body.splice(exportDefaultDeclarationIndex, 0, {
            type: 'VariableDeclaration',
            kind: 'var',
            declarations: [{
              type: 'VariableDeclarator',
              id: {
                type: 'Identifier',
                name: _shared.STORY_GROUP_DEFAULT_EXPORT_IDENTIFIER
              },
              init: exportDefaultDeclarationNode.declaration
            }]
          }); // Adds story group parameter, containing array of all source files

          var addStoryGroupParamStatements = (0, _parser.parse)("\n            ".concat(_shared.STORY_GROUP_DEFAULT_EXPORT_IDENTIFIER, ".parameters = ").concat(_shared.STORY_GROUP_DEFAULT_EXPORT_IDENTIFIER, ".parameters || {};\n            ").concat(_shared.STORY_GROUP_DEFAULT_EXPORT_IDENTIFIER, ".parameters.").concat(_shared.PARAM_KEY, " = {\n              sourceFiles: [\n                ").concat(sourceFiles.map(function (_ref2) {
            var path = _ref2.path,
                id = _ref2.id;
            return "{\n                    path: ".concat(JSON.stringify(path), ",\n                    code: ").concat(id, "\n                  }");
          }), "\n              ]\n            };\n          ")).program.body;

          (_node$body = node.body).splice.apply(_node$body, [exportDefaultDeclarationIndex + 2, 0].concat(_toConsumableArray(addStoryGroupParamStatements)));
        } else {// TODO
        }

        return node;
      } else if (node.type === 'ExportDefaultDeclaration') {
        // Replaces the default export declaration with the variable identifier
        node.declaration = {
          type: 'Identifier',
          name: _shared.STORY_GROUP_DEFAULT_EXPORT_IDENTIFIER
        };
        return node;
      }
    }
  });
}

function _default(source) {
  var ast = (0, _parser.parse)(source, {
    sourceType: 'module'
  });
  var sourceFiles = getSourceFiles(ast);
  addStoryGroupParam(ast, {
    sourceFiles: sourceFiles
  });
  var sourceFileImportStatements = sourceFiles.map(function (_ref3) {
    var path = _ref3.path,
        id = _ref3.id;
    return "import ".concat(id, " from ").concat(JSON.stringify("!!raw-loader!".concat(path)));
  });
  return "\n    ".concat(sourceFileImportStatements.join('\n'), "\n    ").concat((0, _generator["default"])(ast).code, "\n  ");
}